import re
from .music_theory import KEYS_SHARP, KEYS_FLAT, MODE, INCREMENT, hasChords, isMusical

class Transposer:
    def __init__(self, document=None, origKey='A', targetKey='A'):
        self.document = document
        self.origKey = origKey
        self.targetKey = targetKey

    def chordToChord(self, chord, increment):
        if chord in KEYS_SHARP:
            j = KEYS_SHARP.index(chord)
            return KEYS_SHARP[(j+increment)%12]
        j = KEYS_FLAT.index(chord)
        return KEYS_FLAT[(j+increment)%12]

    def transpose(self, chord, increment):
        chords = chord.split('/');

        firstChord = self.chordToChord(chords[0], increment)

        if len(chords) == 2:
            return "{}/{}".format(firstChord, self.chordToChord(chords[1], increment))
        return firstChord

    def parse(self):
        origKeyId = KEYS_SHARP.index(self.origKey)
        targetKeyId = KEYS_SHARP.index(self.targetKey)

        increment = (targetKeyId-origKeyId)%12

        for i in range(len(self.document.paragraphs)):
            p = self.document.paragraphs[i]
            inline = p.runs
            for j in range(len(inline)):
                inline[j].text = re.sub(r"(?<![A-Za-z])([A-G](?![A-Zac-ln-z])[#b]?)", lambda chord: self.transpose(chord.group(0), increment), inline[j].text)



import re

KEYS_SHARP = ['A', 'A#', 'B', 'C', 'C#', 'D', 'D#', 'E', 'F', 'F#', 'G', 'G#']
KEYS_FLAT = ['A', 'Bb', 'B', 'C', 'Db', 'D', 'Eb', 'E', 'F', 'Gb', 'G', 'Ab']
MODE = ['', 'm', 'm', '', '', 'm', 'dim']
INCREMENT = [0, 2, 4, 5, 7, 9, 11];

def hasChords(line):
    return bool(re.search(r"[A-G](?![A-Zac-z])[#b]?[Mm]?", line))

def isMusical(line):
    return bool(re.search(r"[A-G](?![A-Zac-z])[#b]?[Mm]?", line)) or bool(re.search(r"\+[0-12]", line))

def isSection(line):
    lineUpper = line.upper()
    sections = ["INTRO", "TAG", "INSTRUMENTAL", "VERSE", "CHORUS", "BRIDGE"]
    for section in sections:
        if section in lineUpper:
            return True
    return False



import re
from docx import Document
from docx.shared import RGBColor, Pt
from .music_theory import isMusical, isSection, hasChords

class DocumentHandler:
    def __init__(self, path):
        with open(path, 'rb') as f:
            self.document = Document(f)

    def save(self, filename):
        self.document.save(filename)



from setuptools import setup

setup(name='Chord123',
      version='0.16',
      description='A command line script to translate a .docx of number-system chords to a specified key',
      author='Callistus Tan',
      author_email='callistusystan@example.com',
      packages=['chord123'],
      keywords=['music', 'translate', 'chords', 'numbers'],
      url='https://github.com/callistusystan/Chord123',
      download_url='https://github.com/callistusystan/Chord123/archive/0.16.tar.gz',
      license='MIT',
      entry_points = {
        "console_scripts": ['chord123 = chord123.chord123:main']
      },
      install_requires=[
        'python-docx',
      ],
      zip_safe=False)
